#ifdef __APPLE__
#import <AvailabilityMacros.h>
#else
#include <math.h>
#ifndef NAN
#define NAN FP_NAN
#endif
#endif

#import <Foundation/Foundation.h>

#import <stdio.h>
#import <math.h>
#import <assert.h>
#import <string.h>

#import <pixel.h>
#import <material.h>

//---- @implementation section ----

@implementation material_t

- (id) init
{
  if(!(self = [super init]))
    return nil;
  cookie = MAT_COOKIE;
  name = [[NSString alloc] init];
  ambient = [[pixel_t alloc] init];
  diffuse = [[pixel_t alloc] init];
  specular = [[pixel_t alloc] init];
  return self;
}

- (NSString *) setname: (NSString *) _name
{
  /* C++-style
  if(name != _name) {
    // release old obj
    [name release];
    name = [_name retain];
  }
  */
  /* Objective C style */
  // release old obj but only after we retain new one (ok if name == _name)
  [name autorelease];
  name = [_name retain];
}
- (pixel_t *)  setambn: (pixel_t *) _ambn
{
  [ambient autorelease];
  ambient = [_ambn retain];
}
- (pixel_t *)  setdiff: (pixel_t *) _diff
{
  [diffuse autorelease];
  diffuse = [_diff retain];
}
- (pixel_t *)  setspec: (pixel_t *) _spec
{
  [specular autorelease];
  specular = [_spec retain];
}

- (NSString *) getname { return name; }
- (pixel_t *)  getambn { return ambient; }
- (pixel_t *)  getdiff { return diffuse; }
- (pixel_t *)  getspec { return specular; }
- (int)  getcookie { return cookie; }

  // operators

  // members
- (bool) matches: (NSString *) _name
{
  return [name isEqualToString: _name];
}

- (void) read: (FILE *) _in
{
	char	c, _name[256];
	int	count;

  // assume file pointer has already consumed 'material' token
  assert((count = fscanf(_in,"%s",_name))==1);
  [self setname: [[NSString alloc] initWithUTF8String: _name]];

  // consume all chars until we get to '{'
  while((c=fgetc(_in)) != EOF && c != '{');

  // loop until we hit '}'
  while((c=fgetc(_in)) != EOF && c != '}') {
    // put character back into input stream
    fputc(c,_in);

    assert((count = fscanf(_in,"%s",_name))==1);

    if(!strcmp(_name,"ambient")) [ambient read: _in];
    if(!strcmp(_name,"diffuse")) [diffuse read: _in];
    if(!strcmp(_name,"specular")) [specular read: _in];

    // consume whitespace
    while((c=fgetc(_in)) != EOF && c != '\n');
  }
}

- (void) write: (FILE *) _out
{
  assert(cookie == MAT_COOKIE);
 
  fprintf(_out,"material %s\n",[name UTF8String]);
  fprintf(_out,"{\n");
  if([ambient nonzero]) [ambient write: _out: "   ambient"];
  if([diffuse nonzero]) [diffuse write: _out: "   diffuse"];
  if([specular nonzero]) [specular write: _out: "  specular"];
  fprintf(_out,"}\n\n");
}

@end
