#ifdef __APPLE__
#import <AvailabilityMacros.h>
#else
#include <math.h>
#ifndef NAN
#define NAN FP_NAN
#endif
#endif

#import <Foundation/Foundation.h>

#import <stdio.h>
#import <math.h>

#import <pixel.h>

//---- @implementation section ----

@implementation pixel_t

- (id) init { return [self init: 0.0: 0.0: 0.0]; }
- (id) init: (double) r { return [self init: r: 0.0: 0.0]; }
- (id) init: (double) r: (double) g { return [self init: r: g: 0.0]; }
- (id) init: (double) r: (double) g: (double) b
{
  if(!(self = [super init]))
    return nil;
  drgb[0] = r;  drgb[1] = g;  drgb[2] = b;
  return self;
}

//---- @protcols section ----
- (id) copyWithZone: (NSZone *) zone
{
  pixel_t *clone = [[pixel_t allocWithZone: zone] init];
  [clone set: drgb[0]: drgb[1]: drgb[2]];
  return clone;
}

- (bool) nonzero
{
	int	i;
	double	precision=0.000001;

  for(i=0;i<3;i++) if(fabs(drgb[i]) > precision) return true;

  return false;
}
- (double) get: (int) i { return drgb[i]; }
- (void) set: (int) i: (double) d { drgb[i] = d; }
- (void) set: (double) r: (double) g: (double) b
{
  drgb[0] = r;  drgb[1] = g;  drgb[2] = b;
}

- (pixel_t *) add: (pixel_t *) p
{
	pixel_t	*result = [[pixel_t alloc] init];
	int	i;

  for(i=0;i<3;i++) [result set: i: drgb[i] + [p get: i]];

  // avoid memory leak by releasing result after use (unless retain'ed)
  return [result autorelease];
}

- (pixel_t *) dif: (pixel_t *) p
{
	pixel_t	*result = [[pixel_t alloc] init];
	int	i;

  for(i=0;i<3;i++) [result set: i: drgb[i] - [p get: i]];

  // avoid memory leak by releasing result after use (unless retain'ed)
  return [result autorelease];
}

- (pixel_t *) scale: (double) d
{
	pixel_t	*result = [[pixel_t alloc] init];
	int	i;

  // avoid memory leak by releasing result after use (unless retain'ed)
  for(i=0;i<3;i++) [result set: i: drgb[i] * d];

  return [result autorelease];
}

- (void) read: (FILE *) _in
{
	int	i;
	int	c=0;

  for(i=0;i<3;i++) c += fscanf(_in,"%lf",&drgb[i]);
}

- (void) write: (FILE *) _out: (char *) _msg
{	
	int	i;

  fprintf(_out,"%s",_msg);
  for(i=0; i<3; i++)
    drgb[i] < 0.0 ?  fprintf(_out,"%3.0lf",drgb[i]) :
                     fprintf(_out,"%2.0lf",drgb[i]);
  fprintf(_out,"\n");
}

@end
